/* artistEntry.C

   Copyright (C) 1999 by David Rose.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   To contact the original author of this program, write to:
           David Rose
           224 W. Dryden St Apt. 418
	   Glendale, CA 91202
	   USA

	   drose@lafn.org
   */

#include "system.h"

#include "artistEntry.h"
#include "artist.h"
#include "artistDialog.h"
#include "artistLookup.h"

#define XK_MISCELLANY
#include <X11/keysymdef.h>

ArtistEntry::
ArtistEntry(const Dataset &data) : 
  _data(data),
  _edit_button("Edit") 
{
  _dialog = NULL;
  _lookup = NULL;
  
  layout_window();
}

ArtistEntry::
~ArtistEntry() {
  delete_dialog();
  delete_lookup();
}

void ArtistEntry::
set_artist(const TableDataProxy &artist) {
  _data[TI_artists] = artist;
  _orig_name = get_formatted_artist_name(artist);
  _entry.set_text(_orig_name);
}

const TableDataProxy &ArtistEntry::
get_artist() {
  // Check to see if the user has changed the artist's name.  If s/he
  // has, that defines a totally new artist.
  string name = _entry.get_text();
  if (_orig_name != name) {
    set_artist(find_artist(name));
  }
  return _data[TI_artists];
}

void ArtistEntry::
popup_dialog() {
  delete_dialog();
  _dialog = new ArtistDialog(_data, -1);
  connect_to_method(_dialog->deleted, this, &dialog_deleted);
  connect_to_method(_dialog->accepted, this, &dialog_accepted);
}

void ArtistEntry::
delete_dialog() {
  if (_dialog != NULL) {
    delete _dialog;
    _dialog = NULL;
  }
}

void ArtistEntry::
popup_lookup() {
  delete_lookup();
  _lookup = new ArtistLookup(_data, false);
  connect_to_method(_lookup->deleted, this, &lookup_deleted);
  connect_to_method(_lookup->accepted, this, &lookup_accepted);
}

void ArtistEntry::
delete_lookup() {
  if (_lookup != NULL) {
    _lookup->hide();
    delete _lookup;
    _lookup = NULL;
  }
}

void ArtistEntry::
layout_window() {
  pack_start(_entry, true, true, 0);
  _entry.show();
  connect_to_method(_entry.activate, this, &entry_activated);
  connect_to_method(_entry.key_press_event, this, &entry_keypress);

  pack_start(_edit_button, false, false, 4);
  _edit_button.set_usize(40, 24);
  _edit_button.show();
  connect_to_method(_edit_button.clicked, this, &edit_button_clicked);
}

// When we click the edit button, or press F1, it means to pop up the
// edit-artist dialog.
void ArtistEntry::
edit_button_clicked() {
  get_artist();
  popup_dialog();
}

void ArtistEntry::
entry_activated() {
  get_artist();
  activate();
}

gint ArtistEntry::
entry_keypress(GdkEventKey *p1) {
  switch (p1->keyval) {
  case XK_F1:
    // F1 : Edit artist.
    popup_dialog();
    return true;

  case XK_F2:
    // F2 : Look up another artist.
    popup_lookup();
    return true;
  }

  return false;
}

// Called on return from the ArtistDialog, created via popup_dialog().
void ArtistEntry::
dialog_accepted() {
  set_artist(_dialog->get_data()[TI_artists]);
}

void ArtistEntry::
dialog_deleted() {
  _dialog = NULL;
}

// Called on return from the ArtistLookup, created via popup_lookup().
void ArtistEntry::
lookup_accepted() {
  set_artist(_lookup->get_artist());
}

void ArtistEntry::
lookup_deleted() {
  _lookup = NULL;
}

