/* cdTrackPlayer.C

   Copyright (C) 1999 by David Rose.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   To contact the original author of this program, write to:
           David Rose
           224 W. Dryden St Apt. 418
	   Glendale, CA 91202
	   USA

	   drose@lafn.org
   */

#include "system.h"

#include "cdTrackPlayer.h"

static const int cd_frames_per_second = 75;

#ifdef CDAUDIO_OLD
  #define TOTALTRACKS _disc.disc_totaltracks
  #define TRACK track
#else
  #define TOTALTRACKS _disc.disc_total_tracks
  #define TRACK disc_track
#endif

CDTrackPlayer::
CDTrackPlayer(const string &device_name, int track_num) {
  _is_valid = false;
  _cd_desc = cd_init_device((char *)device_name.c_str());
  _track_index = -1;

  if(_cd_desc < 0) {
    if(errno == EBUSY) {
      cerr << "Cannot read " << device_name 
	   << " as it is currently mounted.\n";
    } else {
      perror(device_name.c_str());
    }

    return;
  }
  
  cd_stat(_cd_desc, &_disc);

  if (!_disc.disc_present) {
    cd_stat(_cd_desc, &_disc);
    if (!_disc.disc_present) {
      cerr << "No disc in drive.\n";
      return;
    }
  }

  if (track_num < 1 || track_num > TOTALTRACKS) {
    cerr << "Invalid track number.\n";
    return;
  }

  _track_index = track_num - 1;
  _first_frame = get_start_frame(_track_index);
  _last_frame = get_end_frame(_track_index);
  _is_valid = (_last_frame > _first_frame);
}

CDTrackPlayer::
~CDTrackPlayer() {
  stop();
}

bool CDTrackPlayer::
is_valid() const {
  return _is_valid;
}

double CDTrackPlayer::
get_length() const {
  return (double)get_track_length(_track_index);
}

double CDTrackPlayer::
get_current_time() {
  cd_stat(_cd_desc, &_disc);
#ifdef CDAUDIO_OLD
  int frame = _disc.disc_frame;
#else
  int frame = _disc.disc_current_frame;
#endif

  return (double)(frame - _first_frame) / (double)cd_frames_per_second;
}
  
void CDTrackPlayer::
play(double start, double length) {
  int start_frame = (int)(start * cd_frames_per_second + _first_frame);
  int stop_frame = _last_frame;

  if (length >= 0.0) {
    stop_frame = start_frame + (int)(length * cd_frames_per_second);
  }

  cd_play_frames(_cd_desc, start_frame, stop_frame);
}

void CDTrackPlayer::
stop() {
  cd_stop(_cd_desc);
}

bool CDTrackPlayer::
is_playing() {
  cd_stat(_cd_desc, &_disc);
  return _disc.disc_mode == CDAUDIO_PLAYING;
}

int CDTrackPlayer::
get_start_frame(int track_index) const {
  return _disc.TRACK[track_index].track_start;
}

int CDTrackPlayer::
get_end_frame(int track_index) const {
  if (track_index + 1 < TOTALTRACKS) {
    return _disc.TRACK[track_index + 1].track_start - 1;
  } else {
    // Hmm, a limitation with the cdaudio library--we can't get the
    // last frame on the CD.  We'll estimate from the time in seconds.
    return (_disc.TRACK[track_index].track_start + 
	    get_track_length(track_index) * cd_frames_per_second);
  }
}

int CDTrackPlayer::
get_track_length(int track_index) const {
  assert(track_index >= 0 && track_index < TOTALTRACKS);
 
  return _disc.TRACK[track_index].track_length.minutes * 60 +
    _disc.TRACK[track_index].track_length.seconds;
}
